#!/bin/bash
# Convert large prob files to plink
# Uses a modified version of prob2plink.pl
# Arjen van Bochoven and Danielle Posthuma
# 2011-12-07

# Default path to prob2plinkbig.pl
PROB2PLINK='./prob2plinkbig.pl'

# Default output file prefix
OUT='recoded'

# Default chunk size in lines
LINES=100

VERBOSE=

usage()
{
cat << EOF
usage: $0 options

This script converts MACH probability and info files to plink dosage input format.
It uses a slightly adjusted version of prob2plink.pl written by Yun Li.
Note: when running this script on multiple chromosomes, make sure to
create separate directories for each chromosome as the script generates 
multiple temporary files

OPTIONS:
   -p      Path to prob file (required)
   -i      Path to info file (required)
   -o      Output prefix (default 'recoded')
   -l      Chunk size in lines (default '100')
   -s      Path to prob2plinkbig.pl (default ./prob2plinkbig.pl)
   -v      Verbose
   -h      Show this message
EOF
}

# Get options
while getopts 'p:i:o:s:l:v' OPTION
do
	case $OPTION in
		h)
			usage
			exit 1
			;;
		p)
			PROB=$OPTARG
			;;
		i)
			INFO=$OPTARG
			;;
		o)
			OUT=$OPTARG
			;;
		s)
			PROB2PLINK=$OPTARG
			;;
		l)
			LINES=$OPTARG
			;;
		v)
			VERBOSE=1
			;;
		?)
			usage
			exit
			;;
	esac
done

if [[ -z $PROB ]] || [[ -z $INFO ]]
then
     usage
     exit 1
fi

# Check if prob2plinkbig.pl exists
if [ ! -e $PROB2PLINK ]; then
	echo "prop2plinkbig.pl not found (tried $PROB2PLINK)"
	exit 1
fi

# Check prob2plink nohead support
if [ $(grep -c nohead $PROB2PLINK) == '0'  ]; then
	echo 'Error: wrong version of prob2plink.pl, this script needs nohead support'
	exit 1
fi

if [ "$VERBOSE" == '1' ]; then echo "## Splitting file in $LINES line chunks.."; fi

# Split file into smaller chunks
gunzip -c $PROB | split -l $LINES -a 3

if [ "$VERBOSE" == '1' ]; then echo '## Processing chunks..'; fi


# Process first chunk with headers
$PROB2PLINK -prob xaaa -info $INFO -o cxaaa
# Remove first chunk
rm xaaa

# Check if there are more chunks left
if [ -f xaab ]; then

	# Process remaining chunks
	for i in x*; 
		do $PROB2PLINK -prob $i -info $INFO -o c$i -nohead 1;
		rm $i;
	done
fi

if [ "$VERBOSE" == '1' ]; then echo '## Building .plink_dat..'; fi


# Paste plink_dat files together
paste -d '\0' cx*.plink_dat > $OUT.plink.plink_dat

if [ "$VERBOSE" == '1' ]; then echo '## Building .fam..'; fi

# Create .fam file
cat cx*.fam > $OUT.plink.fam

if [ "$VERBOSE" == '1' ]; then echo '## Cleaning up..'; fi

# Clean up converted chunks
rm cx*

if [ "$VERBOSE" == '1' ]; then echo '## Done'; fi
